<?php

/*
 * @author     M2E Pro Developers Team
 * @copyright  2011-2016 ESS-UA [M2E Pro]
 * @license    Commercial use is forbidden
 */

namespace Ess\M2eProUpdater\Block\Adminhtml\Configuration;

use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Setup\Model\Cron\ReadinessCheck;
use Magento\Setup\Model\CronScriptReadinessCheck;
use Magento\Framework\Data\Form\Element\Renderer\RendererInterface;
use Ess\M2eProUpdater\Model\Cron\Task\AbstractModel;
use Ess\M2eProUpdater\Block\Adminhtml\AbstractBlock;
use Ess\M2eProUpdater\Model\LoggerFactory;
use Magento\Framework\View\Element;

class Messages extends AbstractBlock implements RendererInterface
{
    /** @var \Magento\Framework\Filesystem $fileSystem */
    protected $fileSystem;

    /** @var \Magento\Framework\Filesystem\Directory\ReadFactory */
    protected $directoryReaderFactory;

    /** @var \Magento\Framework\Stdlib\DateTime\DateTime */
    private $localeDate;

    //########################################

    public function __construct(
        \Magento\Framework\Filesystem\Directory\ReadFactory $directoryReaderFactory,
        \Ess\M2eProUpdater\Model\LoggerFactory $loggerFactory,
        \Magento\Framework\Stdlib\DateTime\DateTime $localeDate,
        \Ess\M2eProUpdater\Helper\Factory $helperFactory,
        \Ess\M2eProUpdater\Model\Factory $modelFactory,
        \Magento\Backend\Block\Template\Context $context,
        array $data = []
    ){
        $this->directoryReaderFactory = $directoryReaderFactory;
        $this->fileSystem             = $context->getFilesystem();
        $this->localeDate             = $localeDate;

        parent::__construct($helperFactory, $modelFactory, $context, $data);
    }

    //########################################

    public function render(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        return $this->_toHtml();
    }

    //########################################

    protected function _toHtml()
    {
        /** @var \Magento\Framework\View\Element\Messages $messagesBlock */
        $messagesBlock = $this->getLayout()->createBlock('Magento\Framework\View\Element\Messages');

        if ($this->helperFactory->getObject('M2ePro')->isInstalledViaComposer()) {
            $this->addInstalledViaComposerMessage($messagesBlock);
            return $messagesBlock->toHtml();
        }

        $this->addCronMessages($messagesBlock);
        $this->addScheduleMessage($messagesBlock);

        $this->addMagentoCronMessages($messagesBlock);
        $this->addMagentoUpdateCronMessages($messagesBlock);

        $this->addGlobalCronErrorMessage($messagesBlock);
        $this->addLatestUpgradeWasFailedMessage($messagesBlock);

        return $messagesBlock->toHtml();
    }

    //########################################

    private function addInstalledViaComposerMessage(Element\Messages $messagesBlock)
    {
        $message = <<<HTML
The Module was initially installed via Composer. 
To avoid technical issues in the system, the M2E Pro upgrade cannot be performed via the Updater.
HTML;
        $messagesBlock->addWarning(__($message));
    }

    private function addCronMessages(Element\Messages $messagesBlock)
    {
        /** @var \Ess\M2eProUpdater\Helper\Cron $helper */
        $helper = $this->helperFactory->getObject('Cron');

        $url = $this->helperFactory->getObject('Support')->getDocumentationUrl(null, 'x/bQM0AQ');
        $url .= '#viaM2EProInstallation/UpgradeModule-SetupInstallation\UpgradeModule';
        if (!$helper->isInstalled()) {

            $message = <<<HTML
Attention! Cron file <i>m2epro_updater_cron.php</i> does not work. 
Make sure you complete all the steps of <a target="_blank" href="{$url}">set-up instructions</a> 
for M2E Pro Updater.
HTML;
            $messagesBlock->addError(__($message));
            return;
        }

        if ($helper->isLastRunMoreThan(1800)) {

            $message = <<<HTML
Attention! Cron file <i>m2epro_updater_cron.php</i> has not worked for more than 30 minutes. 
Make sure you complete all the steps of <a target="_blank" href="{$url}">set-up instructions</a> 
for M2E Pro Updater.
HTML;
            $messagesBlock->addWarning(__($message));
        }
    }

    //----------------------------------------

    private function addMagentoCronMessages(Element\Messages $messagesBlock)
    {
        $directory = $this->fileSystem->getDirectoryWrite(DirectoryList::VAR_DIR);
        $url = 'http://devdocs.magento.com/guides/v2.0/comp-mgr/prereq/prereq_cron.html';

        if (!$directory->isExist(ReadinessCheck::SETUP_CRON_JOB_STATUS_FILE)) {
            $message = <<<HTML
Attention! Cron command <i>bin/magento setup:cron:run</i> does not work. 
Make sure you complete <a target="_blank" href="{$url}">these instructions</a>.
HTML;
            $messagesBlock->addError(__($message));
            return;
        }

        $lastRunStamp = filemtime($directory->getAbsolutePath(ReadinessCheck::SETUP_CRON_JOB_STATUS_FILE));
        if ($this->localeDate->gmtTimestamp() > $lastRunStamp + 1800) {
            $message = <<<HTML
Attention! Cron command <i>bin/magento setup:cron:run</i> has not worked for more than 30 minutes. 
Make sure you complete <a target="_blank" href="{$url}">these instructions</a>.
HTML;
            $messagesBlock->addWarning(__($message));
        }
    }

    private function addMagentoUpdateCronMessages(Element\Messages $messagesBlock)
    {
        $directory = $this->fileSystem->getDirectoryWrite(DirectoryList::VAR_DIR);
        $url = 'http://devdocs.magento.com/guides/v2.0/comp-mgr/prereq/prereq_cron.html';

        if (!$directory->isExist(CronScriptReadinessCheck::UPDATER_CRON_JOB_STATS_FILE)) {
            $message = <<<HTML
Attention! Cron file <i>update/cron.php</i> does not work.
Make sure you complete <a target="_blank" href="{$url}">these instructions</a>.
HTML;
            $messagesBlock->addError(__($message));
            return;
        }

        $lastRunStamp = filemtime($directory->getAbsolutePath(CronScriptReadinessCheck::UPDATER_CRON_JOB_STATS_FILE));
        if ($this->localeDate->gmtTimestamp() > $lastRunStamp + 1800) {
            $message = <<<HTML
Attention! Cron file <i>update/cron.php</i> has not worked for more than 30 minutes. 
Make sure you complete <a target="_blank" href="{$url}">these instructions</a>.
HTML;
            $messagesBlock->addWarning(__($message));
        }

        $content = (array)json_decode($directory->readFile(CronScriptReadinessCheck::UPDATER_CRON_JOB_STATS_FILE), true);
        if (!empty($content[ReadinessCheck::KEY_READINESS_CHECKS]['error'])) {
            $errorsFile = $directory->getRelativePath(CronScriptReadinessCheck::UPDATER_CRON_JOB_STATS_FILE);
            $error = $content[ReadinessCheck::KEY_READINESS_CHECKS]['error'];
            $message = <<<HTML
Failed to install/upgrade M2E Pro because the file <b>{$errorsFile}</b> has errors:
{$error}
HTML;
            $messagesBlock->addWarning(__($message));
        }
    }

    //----------------------------------------

    private function addGlobalCronErrorMessage(Element\Messages $messagesBlock)
    {
        if ($this->isGlobalCronErrorMessages()) {

            $fileName = LoggerFactory::LOGFILE_NAME;
            $downloadUrl = $this->_urlBuilder->getUrl('M2eProUpdater/log/get', ['file_name' => $fileName]);
            $removeUrl   = $this->_urlBuilder->getUrl('M2eProUpdater/log/remove', ['file_name' => $fileName]);

            $message = <<<HTML
Failed to install/upgrade M2E Pro. View <a href="{$downloadUrl}">logs</a> for the details.
Click <a href="{$removeUrl}">here</a> to ignore this message. 
HTML;
            $messagesBlock->addError(__($message));
        }
    }

    private function isGlobalCronErrorMessages()
    {
        /** @var \Ess\M2eProUpdater\Helper\Module $moduleHelper */
        $moduleHelper = $this->helperFactory->getObject('Module');
        $directory = $this->directoryReaderFactory->create($moduleHelper->getLogDirectoryPath());

        if (!$directory->isExist(LoggerFactory::LOGFILE_NAME)) {
            return false;
        }

        return true;
    }

    //----------------------------------------

    private function addLatestUpgradeWasFailedMessage(Element\Messages $messagesBlock)
    {
        if ($this->isLatestUpgradeWasCompletedUnsuccessfully()) {

            $fileName = $this->getLatestUpgradeLogFileName();
            $downloadUrl = $this->_urlBuilder->getUrl('M2eProUpdater/log/get', ['file_name' => $fileName]);
            $removeUrl   = $this->_urlBuilder->getUrl('M2eProUpdater/log/remove', ['file_name' => $fileName]);

            $message = <<<HTML
Failed to install/upgrade M2E Pro. View <a href="{$downloadUrl}">logs</a> for the details.
Click <a href="{$removeUrl}">here</a> to ignore this message.
HTML;
            $messagesBlock->addError(__($message));
        }
    }

    private function addScheduleMessage(Element\Messages $messagesBlock)
    {
        /** @var \Ess\M2eProUpdater\Helper\Config $configHelper */
        $configHelper = $this->helperFactory->getObject('Config');
        if (!$configHelper->isPrepareUpgradeTaskAllowed() && !$configHelper->isDoUpgradeTaskAllowed()) {
            return;
        }

        $message = $this->helperFactory->getObject('M2ePro')->isInstalled()
            ? __('M2E Pro will be upgraded shortly. Please wait.')
            : __('M2E Pro will be installed shortly. Please wait.');

        $messagesBlock->addNotice($message);
    }

    private function isLatestUpgradeWasCompletedUnsuccessfully()
    {
        /** @var \Ess\M2eProUpdater\Helper\Module $moduleHelper */
        $moduleHelper = $this->helperFactory->getObject('Module');

        $filename = $this->getLatestUpgradeLogFileName();
        $directory = $this->directoryReaderFactory->create($moduleHelper->getLogDirectoryPath());

        if (!$directory->isExist($filename)) {
            return false;
        }

        return true;
    }

    private function getLatestUpgradeLogFileName()
    {
        /** @var \Ess\M2eProUpdater\Helper\M2ePro $m2eProHelper */
        $m2eProHelper = $this->helperFactory->getObject('M2ePro');
        $latestVersion  = $m2eProHelper->getLatestAvailableVersion();

        $fileName = AbstractModel::LOG_FILE_NAME_MASK;
        $latestVersion && $fileName = str_replace('%ver%', $latestVersion, $fileName);

        return $fileName;
    }

    //########################################
}