<?php

/*
 * @author     M2E Pro Developers Team
 * @copyright  2011-2016 ESS-UA [M2E Pro]
 * @license    Commercial use is forbidden
 */

namespace Ess\M2eProUpdater\Helper;

use Magento\Framework\App\Filesystem\DirectoryList;

class M2ePro extends \Ess\M2eProUpdater\Helper\AbstractHelper
{
    const IDENTIFIER = 'Ess_M2ePro';

    const RELEASES_INFO_CACHE_KEY       = '_m2epro_releases_info_cache_';
    const LATEST_RELEASE_INFO_CACHE_KEY = '_m2epro_latest_release_info_cache_';

    /** @var \Magento\Framework\Module\FullModuleList */
    protected $fullModuleList;

    /** @var \Magento\Framework\Module\ModuleList */
    protected $moduleList;

    /** @var \Magento\Framework\App\Filesystem\DirectoryList */
    protected $directoryList;

    /** @var \Ess\M2eProUpdater\Model\Factory */
    protected $modelFactory;

    /** @var \Magento\Framework\ObjectManagerInterface  */
    protected $objectManager;

    /** @var \Magento\Framework\Composer\ComposerInformation */
    protected $composerInformation;

    protected $composerPackagesList = [
        'm2e/ebay-amazon-magento2',
        'm2epro/magento2-extension'
    ];

    //########################################

    public function __construct(
        \Magento\Framework\Module\FullModuleList $fullModuleList,
        \Magento\Framework\Module\ModuleList $moduleList,
        \Magento\Framework\App\Filesystem\DirectoryList $directoryList,
        \Ess\M2eProUpdater\Helper\Factory $helperFactory,
        \Ess\M2eProUpdater\Model\Factory $modelFactory,
        \Magento\Framework\ObjectManagerInterface $objectManager,
        \Magento\Framework\Composer\ComposerInformation $composerInformation,
        \Magento\Framework\App\Helper\Context $context
    ){
        $this->fullModuleList      = $fullModuleList;
        $this->moduleList          = $moduleList;
        $this->directoryList       = $directoryList;
        $this->modelFactory        = $modelFactory;
        $this->objectManager       = $objectManager;
        $this->composerInformation = $composerInformation;

        parent::__construct($helperFactory, $context);
    }

    //########################################

    public function isInstalled()
    {
        return $this->fullModuleList->getOne(self::IDENTIFIER);
    }

    public function isInstalledViaComposer()
    {
        foreach ($this->composerPackagesList as $package) {
            if ($this->composerInformation->isPackageInComposerJson($package)) {
                return true;
            }
        }

        return false;
    }

    public function isEnabled()
    {
        return $this->moduleList->getOne(self::IDENTIFIER);
    }

    //----------------------------------------

    public function getCurrentVersion()
    {
        if (!$this->isInstalled()) {
            return NULL;
        }

        /** @var \Ess\M2ePro\Helper\Module $helper */
        $helper = $this->objectManager->get('Ess\M2ePro\Helper\Module');
        return $helper->getPublicVersion();
    }

    public function getLatestAvailableVersion($useCache = true)
    {
        $releaseInfo = $this->getLatestAvailableReleaseInfo($useCache);
        return isset($releaseInfo['version']) ? $releaseInfo['version'] : NULL;
    }

    //----------------------------------------

    public function getCurrentReleaseInfo($useCache = true)
    {
        if (!$this->isInstalled()) {
            return NULL;
        }

        foreach ($this->getReleasesInfo($useCache) as $version => $releaseInfo) {
            if (!empty($releaseInfo['version']) && $this->getCurrentVersion() == $releaseInfo['version']) {
                return $releaseInfo;
            }
        }

        return NULL;
    }

    public function getLatestAvailableReleaseInfo($useCache = true)
    {
        /** @var \Ess\M2eProUpdater\Helper\Data\Cache\Permanent $helper */
        $helper = $this->getHelper('Data\Cache\Permanent');

        $info = NULL;
        if ($useCache && ($info = $helper->getValue(self::LATEST_RELEASE_INFO_CACHE_KEY))) {
            return $info;
        }

        /** @var \Ess\M2eProUpdater\Model\Downloader $downloader */
        $downloader = $this->modelFactory->getObject('Downloader', [
            'visibility'    => $this->getPackageVisibility(),
            'extensionCode' => self::IDENTIFIER
        ]);

        if ($info = $downloader->getReleaseInfo(NULL)) {
            $helper->setValue(self::LATEST_RELEASE_INFO_CACHE_KEY, $info, [], 60*60);
        }

        return $info;
    }

    //----------------------------------------

    public function getReleasesInfo($useCache = true)
    {
        /** @var \Ess\M2eProUpdater\Helper\Data\Cache\Permanent $helper */
        $helper = $this->getHelper('Data\Cache\Permanent');

        $releasesInfo = NULL;
        if ($useCache && ($releasesInfo = $helper->getValue(self::RELEASES_INFO_CACHE_KEY))) {
            return $releasesInfo;
        }

        /** @var \Ess\M2eProUpdater\Model\Downloader $downloader */
        $downloader = $this->modelFactory->getObject('Downloader', [
            'visibility'    => NULL,
            'extensionCode' => self::IDENTIFIER
        ]);

        if ($releasesInfo = $downloader->getReleaseInfo('*')) {
            $helper->setValue(self::RELEASES_INFO_CACHE_KEY, $releasesInfo, [], 60*60);
        }

        return $releasesInfo;
    }

    //----------------------------------------

    public function getPackageVisibility()
    {
        if ($this->isInstalled()) {

            $currentRelease = $this->getCurrentReleaseInfo();
            if (isset($currentRelease['visibility']) &&
                $currentRelease['visibility'] != \Ess\M2eProUpdater\Model\Downloader::VISIBILITY_PUBLIC)
            {
                return $currentRelease['visibility'];
            }
        }

        /** @var \Ess\M2eProUpdater\Helper\Config $config */
        $config = $this->getHelper('Config');
        return $config->getM2eProMinimumStability();
    }

    //########################################

    public function clearCachedReleasesInfo()
    {
        /** @var \Ess\M2eProUpdater\Helper\Data\Cache\Permanent $helper */
        $helper = $this->getHelper('Data\Cache\Permanent');

        $helper->removeValue(self::RELEASES_INFO_CACHE_KEY);
        $helper->removeValue(self::LATEST_RELEASE_INFO_CACHE_KEY);
    }

    //########################################

    public function getCodeDirectoryPath()
    {
        return $this->directoryList->getPath(DirectoryList::APP) . '/code/Ess/M2ePro/';
    }

    //########################################
}