<?php

/*
 * @author     M2E Pro Developers Team
 * @copyright  2011-2016 ESS-UA [M2E Pro]
 * @license    Commercial use is forbidden
 */

namespace Ess\M2eProUpdater\Helper;

use Magento\Framework\App\Filesystem\DirectoryList;

class Module extends \Ess\M2eProUpdater\Helper\AbstractHelper
{
    const IDENTIFIER = 'Ess_M2eProUpdater';

    const RELEASES_INFO_CACHE_KEY       = '_m2epro_updater_releases_info_cache_';
    const LATEST_RELEASE_INFO_CACHE_KEY = '_m2epro_updater_latest_release_info_cache_';

    /** @var \Magento\Framework\App\Filesystem\DirectoryList */
    private $directoryList;

    /** @var \Magento\Framework\Filesystem\Directory\WriteFactory */
    private $directoryWriterFactory;

    /** @var \Magento\Framework\Module\PackageInfo */
    private $packageInfo;

    /** @var \Ess\M2eProUpdater\Model\Factory */
    protected $modelFactory;

    //########################################

    public function __construct(
        \Magento\Framework\App\Filesystem\DirectoryList $directoryList,
        \Magento\Framework\Filesystem\Directory\WriteFactory $directoryWriterFactory,
        \Magento\Framework\Module\PackageInfo $packageInfo,
        \Ess\M2eProUpdater\Helper\Factory $helperFactory,
        \Ess\M2eProUpdater\Model\Factory $modelFactory,
        \Magento\Framework\App\Helper\Context $context
    ) {
        parent::__construct($helperFactory, $context);

        $this->directoryList = $directoryList;
        $this->directoryWriterFactory = $directoryWriterFactory;
        $this->packageInfo = $packageInfo;
        $this->modelFactory = $modelFactory;
    }

    //########################################

    public function getCurrentVersion()
    {
        return $this->packageInfo->getVersion(self::IDENTIFIER);
    }

    public function getLatestAvailableVersion($useCache = true)
    {
        $releaseInfo = $this->getLatestAvailableReleaseInfo($useCache);
        return isset($releaseInfo['version']) ? $releaseInfo['version'] : NULL;
    }

    //----------------------------------------

    public function getCurrentReleaseInfo($useCache = true)
    {
        foreach ($this->getReleasesInfo($useCache) as $version => $releaseInfo) {
            if (!empty($releaseInfo['version']) && $this->getCurrentVersion() == $releaseInfo['version']) {
                return $releaseInfo;
            }
        }

        return NULL;
    }

    public function getLatestAvailableReleaseInfo($useCache = true)
    {
        /** @var \Ess\M2eProUpdater\Helper\Data\Cache\Permanent $helper */
        $helper = $this->getHelper('Data\Cache\Permanent');

        $info = NULL;
        if ($useCache && ($info = $helper->getValue(self::LATEST_RELEASE_INFO_CACHE_KEY))) {
            return $info;
        }

        /** @var \Ess\M2eProUpdater\Model\Downloader $downloader */
        $downloader = $this->modelFactory->getObject('Downloader', [
            'visibility'    => $this->getPackageVisibility(),
            'extensionCode' => self::IDENTIFIER
        ]);

        if ($info = $downloader->getReleaseInfo(NULL)) {
            $helper->setValue(self::LATEST_RELEASE_INFO_CACHE_KEY, $info, [], 60*60);
        }

        return $info;
    }

    //----------------------------------------

    public function getReleasesInfo($useCache = true)
    {
        /** @var \Ess\M2eProUpdater\Helper\Data\Cache\Permanent $helper */
        $helper = $this->getHelper('Data\Cache\Permanent');

        $releasesInfo = NULL;
        if ($useCache && ($releasesInfo = $helper->getValue(self::RELEASES_INFO_CACHE_KEY))) {
            return $releasesInfo;
        }

        /** @var \Ess\M2eProUpdater\Model\Downloader $downloader */
        $downloader = $this->modelFactory->getObject('Downloader', [
            'visibility'    => NULL,
            'extensionCode' => self::IDENTIFIER
        ]);

        if ($releasesInfo = $downloader->getReleaseInfo('*')) {
            $helper->setValue(self::RELEASES_INFO_CACHE_KEY, $releasesInfo, [], 60*60);
        }

        return $releasesInfo;
    }

    //----------------------------------------

    public function getPackageVisibility()
    {
        $currentRelease = $this->getCurrentReleaseInfo();
        if (isset($currentRelease['visibility']) &&
            $currentRelease['visibility'] != \Ess\M2eProUpdater\Model\Downloader::VISIBILITY_PUBLIC)
        {
            return $currentRelease['visibility'];
        }

        /** @var \Ess\M2eProUpdater\Helper\Config $config */
        $config = $this->getHelper('Config');

        return $config->getM2eProMinimumStability();
    }

    //########################################

    public function clearCachedReleasesInfo()
    {
        /** @var \Ess\M2eProUpdater\Helper\Data\Cache\Permanent $helper */
        $helper = $this->getHelper('Data\Cache\Permanent');

        $helper->removeValue(self::RELEASES_INFO_CACHE_KEY);
        $helper->removeValue(self::LATEST_RELEASE_INFO_CACHE_KEY);
    }

    //########################################

    public function getTemporaryDirectoryPath()
    {
        $path = $this->directoryList->getPath(DirectoryList::VAR_DIR) .'/'. self::IDENTIFIER . '/tmp';
        $directory = $this->directoryWriterFactory->create($path);

        if (!$directory->isExist()) {
            $directory->create();
        }

        return $path;
    }

    public function getLogDirectoryPath()
    {
        $path = $this->directoryList->getPath(DirectoryList::VAR_DIR) .'/'. self::IDENTIFIER . '/log';
        $directory = $this->directoryWriterFactory->create($path);

        if (!$directory->isExist()) {
            $directory->create();
        }

        return $path;
    }

    //########################################
}